<?php
/**
* @version $Id: items.php 255 2008-09-04 04:42:00Z jenscski $
* @package MMS Blog
* @copyright Copyright (C) 2006 - 2008 Jens-Christian Skibakk. All rights reserved.
* @license http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
*/

// no direct access
defined( '_JEXEC' ) or die( 'Restricted access' );


jimport( 'joomla.application.component.helper' );
jimport( 'joomla.application.component.model' );

class MMSBlogModelItems extends JModel
{
	var $_data = null;
	var $_total = null;
	var $_pagination = null;

	function __construct()
	{
		parent::__construct();

		global $mainframe;

		// Get the pagination request variables
		$limit      = $mainframe->getUserStateFromRequest( 'global.list.limit', 'limit', $mainframe->getCfg('list_limit'), 'int' );
		$limitstart = $mainframe->getUserStateFromRequest( 'com_mmsblog.users.limitstart', 'limitstart', 0, 'int' );

		$this->setState('limit', $limit);
		$this->setState('limitstart', $limitstart);
	}

	function getData()
	{
		if (empty($this->_data))
		{
			$query = $this->_buildQuery();
			$this->_data = $this->_getList($query, $this->getState('limitstart'), $this->getState('limit'));
		}

		return $this->_data;
	}

	function getTotal()
	{
		if (empty($this->_total))
		{
			$query = $this->_buildQuery();
			$this->_total = $this->_getListCount($query);
		}

		return $this->_total;
	}

	function getPagination()
	{
		if (empty($this->_pagination))
		{
			jimport('joomla.html.pagination');
			$this->_pagination = new Jpagination( $this->getTotal(), $this->getState('limitstart'), $this->getState('limit') );
		}

		return $this->_pagination;
	}

	function _buildQuery()
	{
		$where   = $this->_buildWhere();
		$orderby = $this->_buildOrderBy();

		$query = 'SELECT g.*, u.name, a.name AS accountName, c.title AS categoryName FROM #__mmsblog_item as g'
		. ' LEFT JOIN #__mmsblog_user AS u ON u.id = g.uid'
		. ' LEFT JOIN #__mmsblog_account AS a ON a.id = g.account'
		. ' LEFT JOIN #__categories AS c ON c.id = g.category'
		. $where
		. $orderby
		;

		return $query;
	}

	function _buildWhere()
	{
		global $mainframe;

		$context        = 'com_mmsblog.blogs';
		$filter_state   = $mainframe->getUserStateFromRequest($context.'filter_state',   'filter_state',   '',  'word');
		$filter_catid   = $mainframe->getUserStateFromRequest($context.'filter_catid',   'filter_catid',   '',  'int');
		$filter_account = $mainframe->getUserStateFromRequest($context.'filter_account', 'filter_account', '',  'int');
		$filter_user    = $mainframe->getUserStateFromRequest($context.'filter_user',    'filter_user',    '0', 'int');
		$search         = $mainframe->getUserStateFromRequest($context.'search',         'search',         '',  'string');
		$search         = JString::strtolower( $search );

		$where = array();

		if ($search) {
			$where[] = 'LOWER(g.subject) LIKE '.$this->_db->Quote('%'.$search.'%');
		}

		if ($filter_catid) {
			$where[] = 'g.category='.(int)$filter_catid;
		}

		if ($filter_account) {
			$where[] = 'g.account='.(int)$filter_account;
		}

		if ($filter_user) {
			$where[] = 'g.uid='.(int)$filter_user;
		}

		if ( $filter_state ) {
			if ( $filter_state == 'P' ) {
				$where[] = "g.published = 1";
			} else if ($filter_state == 'U' ) {
				$where[] = "g.published = 0";
			}
		}

		$where = ( count($where) ? ' WHERE ' . implode( ' AND ', $where ) : '' );

		return $where;
	}


	function _buildOrderBy()
	{
		global $mainframe;

		$context          = 'com_mmsblog.items';
		$filter_order     = $mainframe->getUserStateFromRequest( $context.'filter_order',     'filter_order',     'g.ordering', 'cmd' );
		$filter_order_Dir = $mainframe->getUserStateFromRequest( $context.'filter_order_Dir', 'filter_order_Dir', 'desc',       'word' );

		$orderby = ' ORDER BY '.$filter_order.' '.$filter_order_Dir.', g.subject';

		return $orderby;
	}
}

