<?php
/**
* @version $Id: phpimap.php 255 2008-09-04 04:42:00Z jenscski $
* @package MMS Blog
* @copyright Copyright (C) 2006 - 2008 Jens-Christian Skibakk. All rights reserved.
* @license http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
*/

// no direct access
defined( '_JEXEC' ) or die( 'Restricted access' );

class PHPIMAP extends JObject
{
	var $_mailbox = null;
	var $_conn    = null;

	function __construct($params)
	{
		$arr = explode(':', $params->get('server'));

		$server = $arr[0];
		$port   = isset($arr[1]) ? (int)$arr[1] : null;

		$option = $this->_getOption($params->get('option', 0));

		if ($params->get('protocol') == 0) {
			$port = !empty($port) ? (int)$port : $this->_getPort($params->get('option'), 'pop3');
			$this->_mailbox = sprintf('{%s:%d/pop3%s}INBOX', $server, $port, $option);
		} else {
			$port = !empty($port) ? (int)$port : $this->_getPort($params->get('option'), 'imap');
			$this->_mailbox = sprintf('{%s:%d/imap%s}INBOX', $server, $port, $option);
		}
	}

	function &getInstance($params)
	{
		$instance = new PHPIMAP($params);

		$instance->login($params->get('user'), $params->get('password'));

		return $instance;
	}

	function _getPort($option, $proto)
	{
		$ssl = ($option >= 4);

		if ($proto == 'pop3' && $ssl) {
			return 995;
		} elseif ($proto == 'pop3') {
			return 110;
		} elseif ($proto == 'imap' && $ssl) {
			return 993;
		} else {
			return 143;
		}
	}

	function _getOption($option)
	{
		switch($option)
		{
			case 1:
				return '/tls';
				break;

			case 2:
				return '/tls/validate-cert';
				break;

			case 3:
				return '/tls/novalidate-cert';
				break;

			case 4:
				return '/ssl';
				break;

			case 5:
				return '/ssl/validate-cert';
				break;

			case 6:
				return '/ssl/novalidate-cert';
				break;

			case 0:
			default:
				return '/notls';
				break;
		}
	}

	function login($user, $passwd)
	{
		MMSLog::sprintf('Connecting with %s. User: %s', $this->_mailbox, $user);
		$this->_conn = @imap_open($this->_mailbox, $user, $passwd, 0);

		if (!$this->_conn) {
			foreach (imap_errors() as $error) {
				$this->setError($error);
			}
		}
	}

	function close()
	{
		if ($this->_conn)
		{
			$errors = imap_errors(); // clean the error queue
			imap_expunge($this->_conn);
			imap_close($this->_conn);
			$this->_conn = null;
		}
	}

	function getNumMsg()
	{
		return imap_num_msg($this->_conn);
	}


	function &getMsg($msgnum)
	{
		$msg =& MMSMessage::getInstance($this->_conn, $msgnum);

		return $msg;
	}

	function deleteMsg($msgnum)
	{
		imap_delete($this->_conn, $msgnum);
	}
}
