<?php
/**
* @version $Id: updater.php 255 2008-09-04 04:42:00Z jenscski $
* @package MMS Blog
* @copyright Copyright (C) 2006 - 2008 Jens-Christian Skibakk. All rights reserved.
* @license http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
*/

// no direct access
defined( '_JEXEC' ) or die( 'Restricted access' );

JLoader::import('components.com_mmsblog.helpers.helper',     JPATH_SITE);
JLoader::import('components.com_mmsblog.helpers.phpimap',    JPATH_SITE);
JLoader::import('components.com_mmsblog.helpers.mmslog',     JPATH_SITE);
JLoader::import('components.com_mmsblog.helpers.mmsmessage', JPATH_SITE);

jimport('joomla.application.component.helper');
jimport('joomla.error.log');

class MMSUpdater extends JObject
{

	function run($showLog = false)
	{
		global $_PROFILER;

		$app =& JFactory::getApplication();
		$app->set('mmsblog.showlog', $showLog);

		JDEBUG ? $_PROFILER->mark('MMSUpdater::run start') : null;

		MMSLog::_('==== MARK ====');

		$lang =& JFactory::getLanguage();
		$lang->load('com_mmsblog');

		if (!MMSBlogHelper::checkSystem())
		{
			MMSLog::_('...done');
			return;
		}

		$account = MMSUpdater::findAccount();
		if (!$account)
		{
			MMSLog::_('...done');
			return;
		}

		$lock = MMSUpdater::lock($account);
		if (!$lock)
		{
			MMSLog::_('...done');
			return;
		}

		MMSLog::sprintf('Using account: %s', $account->name);

		$params = new JParameter($account->params);
		$con =& PHPIMAP::getInstance($params);

		$updateLock = false;
		if (!$con->getError())
		{
			$config =& JComponentHelper::getParams('com_mmsblog');
			$min = (int)$config->get('download_max', 3);
			$num = $total = (int)$con->getNumMsg();
			$num = min($min, $num);

			MMSLog::sprintf('Found %d messages on server, downloading %d', $total, $num);

			for ($i=0; $i<$num; $i++)
			{
				MMSLog::sprintf('Downloading message %d of %d from server', ($i+1), $num);
				$msg = $con->getMsg($i+1);
				if ($msg)
				{
					MMSLog::sprintf(' - subject: %s', $msg->subject);
					MMSLog::sprintf(' - address: %s', $msg->address);

					$user = MMSBlogHelper::getUser($msg->address);
					MMSLog::sprintf(' - found user: %s', $user->name);

					$category = $config->get('category');
					if ($account->category) {
						$category = $account->category;
					}
					if ($user->get('category')) {
						$category = $user->get('category');
					}

					switch ($user->get('auto_publish'))
					{
						case 1:
							$published = 1;
							break;

						case 2:
							$published = 0;
							break;

						case 0:
						default:
							$published = $config->get('publish_new_content', 1);
						break;
					}

					$row =& JTable::getInstance('mmsitem', 'table');
					$row->set('subject',   $msg->get('subject'));
					$row->set('hash',      $msg->get('hash'));
					$row->set('uid',       $user->get('id'));
					$row->set('address',   $msg->get('address'));
					$row->set('created',   $msg->get('created'));
					$row->set('content',   $msg->get('content'));
					$row->set('account',   $account->id);
					$row->set('published', $published);
					$row->set('category',  $category);

					$row->check();
					if ($row->store())
					{
						MMSUpdater::storeMedia($row, $msg);

						MMSUpdater::newItemMail($row, $user);
						if ($config->get('debug_delete', 1)) {
							MMSLog::_(' - deleting message on server');
							$con->deleteMsg($i+1);
						} else {
							MMSLog::_(' - not deleting message on server');
						}
					}
				}
			}

			$con->close();
			$updateLock = ($total == $num);
		} else {
			// An error occured
			foreach ($con->getErrors() as $error) {
				MMSLog::_('Error: ' . $error);
			}
		}

		MMSUpdater::unlock($account, $updateLock);

		MMSLog::_('...done');
		JDEBUG ? $_PROFILER->mark('MMSUpdater::run end') : null;
	}

	function storeMedia(&$msg, &$media)
	{
		$config =& JComponentHelper::getParams('com_mmsblog');
		$exclude = explode("\n", $config->get('excludefile'));
		foreach ($media->getParts('image') as $part)
		{
			$include = true;
			foreach ($exclude as $pattern) {
				if (trim($pattern) && $part->filename == $pattern) {
					$include = false;
				}
			}

			if ($include)
			{
				$row =& JTable::getInstance('mmsmedia', 'table');
				$row->msgid     = $msg->id;
				$row->filename  = $part->filename;
				$row->type      = $part->type;
				$row->subtype   = $part->subtype;
				$row->published = 1;
				$row->check();
				if ($row->store())
				{
					jimport('joomla.filesystem.file');
					$file = MMSBlogHelper::mediaPath($row);
					JFile::write($file, $part->data);
				}
				MMSLog::sprintf(' - adding image: %s', $part->filename);
			} else {
				MMSLog::sprintf(' - skipping image: %s', $part->filename);
			}
		}

		foreach ($media->getParts('video') as $part)
		{
			$row =& JTable::getInstance('mmsmedia', 'table');
			$row->msgid     = $msg->id;
			$row->filename  = $part->filename;
			$row->type      = $part->type;
			$row->subtype   = $part->subtype;
			$row->published = 1;
			$row->check();
			if ($row->store())
			{
				jimport('joomla.filesystem.file');
				$file = MMSBlogHelper::mediaPath($row);
				JFile::write($file, $part->data);
			}
			MMSLog::sprintf(' - adding video: %s', $part->filename);
		}

		foreach ($media->getParts('audio') as $part)
		{
			$row =& JTable::getInstance('mmsmedia', 'table');
			$row->msgid     = $msg->id;
			$row->filename  = $part->filename;
			$row->type      = $part->type;
			$row->subtype   = $part->subtype;
			$row->published = 1;
			$row->check();
			if ($row->store())
			{
				jimport('joomla.filesystem.file');
				$file = MMSBlogHelper::mediaPath($row);
				JFile::write($file, $part->data);
			}
			MMSLog::sprintf(' - adding audio: %s', $part->filename);
		}
	}

	function newItemMail(&$item, &$user)
	{
		$recipients = array();

		$params =& JComponentHelper::getParams('com_mmsblog');
		$email = $params->get('email', '');

		if (!empty($email)) {
			$recipients[] = $email;
		} else {
			$db =& JFactory::getDBO();
			$query = 'SELECT email FROM #__users WHERE gid=25 AND sendEmail=1';
			$db->setQuery($query);
			$rows = $db->loadObjectList();
			foreach ($rows as $row) {
				$recipients[] = $row->email;
			}
		}

		$url = JURI::root() . 'administrator/index.php?option=com_mmsblog&task=edit&controller=item&cid[]='.$item->id;

		$item_published = $item->published ? JText::_('Yes') : JText::_('No');
		$user_published = $user->published ? JText::_('Yes') : JText::_('No');

		$msg = JText::sprintf('NEWITEMMAIL', $item->subject, $user->name, $item->address, $user_published, $item_published, $url);

		jimport('joomla.mail.helper');
		foreach ($recipients as $recipient)
		{
			$mail =& JFactory::getMailer();
			$mail->addRecipient($recipient);
			$mail->setSubject(JText::_('New MMS Item'));
			$mail->setBody($msg);
			$mail->Send();
		}
	}

	function releaseOrphans()
	{
		$db =& JFactory::getDBO();

		// find orphans
		$query = 'SELECT id FROM #__mmsblog_account e'
		. ' WHERE e.updated < (UNIX_TIMESTAMP()-(60*(e.interval*2)))'
		. ' AND e.session_id != \'\'';

		$db->setQuery($query);
		$rows = $db->loadObjectList();

		if ($rows)
		{
			$cid = array();

			foreach($rows as $row) {
				$cid[] = (int)$row->id;
			}

			$cid = implode(', ', $cid);

			$query = "UPDATE #__mmsblog_account SET session_id='' WHERE id IN ($cid)";
			$db->setQuery($query);
			$db->query();
		}

		MMSLog::sprintf('Released %d orphans', count($rows));
	}

	function unlock($account, $update = true)
	{
		MMSLog::sprintf('Unlocking account: %s', $account->name);
		MMSLog::sprintf(' - update =  %d', $update);

		$db =& JFactory::getDBO();

		if ($update)
		{
			$query = 'UPDATE #__mmsblog_account'
			. ' SET session_id=\'\''
			. ' , updated=UNIX_TIMESTAMP()'
			. ' , lastchecked=UNIX_TIMESTAMP()'
			. ' WHERE id='.(int)$account->id;
		}
		else
		{
			$query = 'UPDATE #__mmsblog_account'
			. ' SET session_id=\'\''
			. ' , lastchecked=UNIX_TIMESTAMP()'
			. ' WHERE id='.(int)$account->id;
		}

		$db->setQuery($query);
		$db->query();
	}

	function lock($account)
	{
		MMSLog::sprintf('Trying to lock account: %s', $account->name);

		$db      =& JFactory::getDBO();
		$session =& JFactory::getSession();

		$query = 'UPDATE #__mmsblog_account'
		. ' SET session_id='.$db->Quote($session->getID())
		. ' WHERE id='.(int)$account->id
		. ' AND session_id=\'\'';

		$db->setQuery($query);
		$db->query();

		$locked = (bool)$db->getAffectedRows();

		if ($locked) {
			MMSLog::sprintf(' - account locked: %s', $account->name);
		} else {
			MMSLog::sprintf(' - account not locked: %s', $account->name);
		}

		return $locked;
	}

	function findAccount()
	{
		MMSUpdater::releaseOrphans();

		$db =& JFactory::getDBO();

		$config =& JComponentHelper::getParams('com_mmsblog');
		$skipInterval = (bool)$config->get('debug_check', 0);

		MMSLog::_('Trying to find a account to check for new messages');
		MMSLog::sprintf(' - skip interval =  %d', $skipInterval);

		if ($skipInterval)
		{
			$query = 'SELECT e.* FROM #__mmsblog_account e'
			. ' WHERE e.published = 1'
			. ' AND e.active = 1'
			. ' AND e.updated < UNIX_TIMESTAMP()'
			. ' AND e.session_id = \'\''
			. ' ORDER BY e.lastchecked LIMIT 1';
		}
		else
		{
			$query = 'SELECT e.* FROM #__mmsblog_account e'
			. ' WHERE e.published=1'
			. ' AND e.active = 1'
			. ' AND e.updated < (UNIX_TIMESTAMP()-(60*e.interval))'
			. ' AND e.session_id = \'\''
			. ' ORDER BY e.lastchecked LIMIT 1';
		}

		$db->setQuery($query);
		$account = $db->loadObject();

		if ($account) {
			MMSLog::sprintf(' - found account: %s', $account->name);
		} else {
			MMSLog::_(' - no account found');
		}

		return $account;
	}
}

